import calls

# Constants
BUY_LIMIT_ORDER_TYPE_BUY = 0
BUY_LIMIT_ORDER_TYPE_SELL = 1

OPEN_ORDERS_TYPE_BUY = 0
OPEN_ORDERS_TYPE_SELL = 1

SELL_LIMIT_ORDER_TYPE_BUY = 0
SELL_LIMIT_ORDER_TYPE_SELL = 1

TRANSACTIONS_SORT_ASCENDING = 'asc'
TRANSACTIONS_SORT_DESCENDING = 'desc'

USER_TRANSACTIONS_SORT_ASCENDING = 'asc'
USER_TRANSACTIONS_SORT_DESCENDING = 'desc'

USER_TRANSACTIONS_TYPE_DEPOSIT = 0
USER_TRANSACTIONS_TYPE_WITHDRAWAL = 1
USER_TRANSACTIONS_TYPE_MARKET_TRADE = 2

WITHDRAWAL_REQUEST_TYPE_SEPA = 0
WITHDRAWAL_REQUEST_TYPE_BITCOIN = 1
WITHDRAWAL_REQUEST_TYPE_WIRE = 2
WITHDRAWAL_REQUEST_TYPE_BITSTAMP_CODE_1 = 3
WITHDRAWAL_REQUEST_TYPE_BITSTAMP_CODE_2 = 4
WITHDRAWAL_REQUEST_TYPE_MTGOX = 5

WITHDRAWAL_REQUEST_STATUS_OPEN = 0
WITHDRAWAL_REQUEST_STATUS_IN_PROCESS = 1
WITHDRAWAL_REQUEST_STATUS_FINISHED = 2
WITHDRAWAL_REQUEST_STATUS_CANCELLED = 3
WITHDRAWAL_REQUEST_STATUS_FAILED = 4


# Wrapper functions
def account_balance(client_id, api_key, api_secret):
    return (
        calls.APIAccountBalanceCall(client_id, api_key, api_secret)
        .call()
    )


def bitcoin_deposit_address(client_id, api_key, api_secret):
    return (
        calls.APIBitcoinDepositAddressCall(client_id, api_key, api_secret)
        .call()
    )


def bitcoin_withdrawal(client_id, api_key, api_secret, amount, address):
    return (
        calls.APIBitcoinWithdrawalCall(client_id, api_key, api_secret)
        .call(amount=amount, address=address)
    )


def buy_limit_order(client_id, api_key, api_secret, amount, price):
    return (
        calls.APIBuyLimitOrderCall(client_id, api_key, api_secret)
        .call(amount=amount, price=price)
    )


def cancel_order(client_id, api_key, api_secret, order_id):
    return (
        calls.APICancelOrderCall(client_id, api_key, api_secret)
        .call(id=order_id)
    )


def check_bitstamp_code(client_id, api_key, api_secret, code):
    return (
        calls.APICheckBitstampCodeCall(client_id, api_key, api_secret)
        .call(code=code)
    )


def eur_usd_conversion_rate():
    return calls.APIEURUSDConversionRateCall().call()


def open_orders(client_id, api_key, api_secret):
    return (
        calls.APIOpenOrdersCall(client_id, api_key, api_secret)
        .call()
    )


def order_book(group=True):
    return calls.APIOrderBookCall().call(group='1' if group else '0')


def redeem_bitstamp_code(client_id, api_key, api_secret, code):
    return (
        calls.APIRedeemBitstampCodeCall(client_id, api_key, api_secret)
        .call(code=code)
    )


def ripple_deposit_address(client_id, api_key, api_secret):
    return (
        calls.APIRippleDepositAddressCall(client_id, api_key, api_secret)
        .call()
    )


def ripple_withdrawal(
        client_id, api_key, api_secret, amount, address, currency):
    return (
        calls.APIRippleWithdrawalCall(client_id, api_key, api_secret)
        .call()
    )


def sell_limit_order(client_id, api_key, api_secret, amount, price):
    return (
        calls.APISellLimitOrderCall(client_id, api_key, api_secret)
        .call(amount=amount, price=price)
    )


def ticker():
    return calls.APITickerCall().call()


def transactions(offset=0, limit=100, sort=TRANSACTIONS_SORT_DESCENDING):
    return calls.APITransactionsCall().call(
        offset=offset, limit=limit, sort=sort
    )


def unconfirmed_bitcoin_deposits(client_id, api_key, api_secret):
    return (
        calls.APIUnconfirmedBitcoinDepositsCall(client_id, api_key, api_secret)
        .call()
    )


def user_transactions(
        client_id, api_key, api_secret, offset=0, limit=100,
        sort=USER_TRANSACTIONS_SORT_DESCENDING):
    return (
        calls.APIUserTransactionsCall(client_id, api_key, api_secret)
        .call(offset=offset, limit=limit, sort=sort)
    )


def withdrawal_requests(client_id, api_key, api_secret):
    return (
        calls.APIWithdrawalRequestsCall(client_id, api_key, api_secret)
        .call()
    )



import datetime
import hashlib
import hmac
import requests
import time
from decimal import Decimal

_API_URL = 'https://www.bitstamp.net/api/'


def dt(timestamp):
    """
    Convert a unix timestamp or ISO 8601 date string to a datetime object.
    """
    if not timestamp:
        return None
    try:
        timestamp = int(timestamp)
    except ValueError:
        try:
            timestamp = time.mktime(
                time.strptime(timestamp, '%Y-%m-%d %H:%M:%S.%f'))
        except ValueError:
            timestamp = time.mktime(
                time.strptime(timestamp, '%Y-%m-%d %H:%M:%S'))
    return datetime.datetime.fromtimestamp(timestamp)


class APIError(Exception):
    pass


class APICall(object):
    url = None
    method = 'get'

    def _process_response(self, response):
        """
        Process the response dictionary.

        If the dictionary is just being altered, then no return is necessary.
        Alternatively, a totally different response can be returned.
        """
        return

    def call(self, **params):
        # Form request
        r = None
        url = _API_URL + self.url
        if self.method == 'get':
            r = requests.get(url, params=params)
        elif self.method == 'post':
            r = requests.post(url, data=params)
        response = r.json()
        # API error?
        if isinstance(response, dict) and 'error' in response:
            raise APIError(response['error'])
        # Process fields
        new_response = self._process_response(response)
        if new_response is not None:
            response = new_response
        return response


class APIPrivateCall(APICall):
    method = 'post'

    def __init__(self, client_id, api_key, api_secret, *args, **kwargs):
        super(APIPrivateCall, self).__init__(*args, **kwargs)
        self.client_id = client_id
        self.api_key = api_key
        self.api_secret = api_secret

    def _get_nonce(self):
        return str(int(time.time()))

    def call(self, **params):
        nonce = self._get_nonce()
        message = nonce + self.client_id + self.api_key
        signature = hmac.new(
            self.api_secret, msg=message, digestmod=hashlib.sha256)
        signature = signature.hexdigest().upper()
        params.update({
            'key': self.api_key, 'signature': signature, 'nonce': nonce
        })
        return super(APIPrivateCall, self).call(**params)


# Specific call classes
class APIAccountBalanceCall(APIPrivateCall):
    url = 'balance/'

    def _process_response(self, response):
        response['btc_reserved'] = Decimal(response['btc_reserved'])
        response['btc_available'] = Decimal(response['btc_available'])
        response['btc_balance'] = Decimal(response['btc_balance'])
        response['usd_reserved'] = Decimal(response['usd_reserved'])
        response['usd_available'] = Decimal(response['usd_available'])
        response['usd_balance'] = Decimal(response['usd_balance'])
        response['fee'] = Decimal(response['fee'])


class APIBitcoinDepositAddressCall(APIPrivateCall):
    url = 'bitcoin_deposit_address/'


class APIBitcoinWithdrawalCall(APIPrivateCall):
    url = 'bitcoin_withdrawal/'


class APIBuyLimitOrderCall(APIPrivateCall):
    url = 'buy/'

    def _process_response(self, response):
        response['datetime'] = dt(response['datetime'])
        response['price'] = Decimal(response['price'])
        response['amount'] = Decimal(response['amount'])


class APICancelOrderCall(APIPrivateCall):
    url = 'cancel_order/'


class APICheckBitstampCodeCall(APIPrivateCall):
    url = 'check_code/'

    def _process_response(self, response):
        response['usd'] = Decimal(response['usd'])
        response['btc'] = Decimal(response['btc'])


class APIEURUSDConversionRateCall(APICall):
    url = 'eur_usd/'

    def _process_response(self, response):
        response['buy'] = Decimal(response['buy'])
        response['sell'] = Decimal(response['sell'])


class APIOrderBookCall(APICall):
    url = 'order_book/'

    def _process_response(self, response):
        response['timestamp'] = dt(response['timestamp'])
        response['bids'] = [{
            'price': Decimal(price),
            'amount': Decimal(amount)
        } for (price, amount) in response['bids']]
        response['asks'] = [{
            'price': Decimal(price),
            'amount': Decimal(amount)
        } for (price, amount) in response['asks']]


class APIOpenOrdersCall(APIPrivateCall):
    url = 'open_orders/'

    def _process_response(self, response):
        for order in response:
            order['datetime'] = dt(order['datetime'])
            order['price'] = Decimal(order['price'])
            order['amount'] = Decimal(order['amount'])


class APIRedeemBitstampCodeCall(APIPrivateCall):
    url = 'redeem_code/'

    def _process_response(self, response):
        response['usd'] = Decimal(response['usd'])
        response['btc'] = Decimal(response['btc'])


class APIRippleDepositAddressCall(APIPrivateCall):
    url = 'ripple_address/'


class APIRippleWithdrawalCall(APIPrivateCall):
    url = 'ripple_withdrawal/'


class APISellLimitOrderCall(APIPrivateCall):
    url = 'sell/'

    def _process_response(self, response):
        response['datetime'] = dt(response['datetime'])
        response['price'] = Decimal(response['price'])
        response['amount'] = Decimal(response['amount'])


class APITickerCall(APICall):
    url = 'ticker/'

    def _process_response(self, response):
        response['last'] = Decimal(response['last'])
        response['high'] = Decimal(response['high'])
        response['low'] = Decimal(response['low'])
        response['volume'] = Decimal(response['volume'])
        response['timestamp'] = dt(response['timestamp'])
        response['bid'] = Decimal(response['bid'])
        response['ask'] = Decimal(response['ask'])


class APITransactionsCall(APICall):
    url = 'transactions/'

    def _process_response(self, response):
        for tx in response:
            tx['date'] = dt(tx['date'])
            tx['price'] = Decimal(tx['price'])
            tx['amount'] = Decimal(tx['amount'])


class APIUnconfirmedBitcoinDepositsCall(APIPrivateCall):
    url = 'unconfirmed_btc/'

    def _process_response(self, response):
        response['amount'] = Decimal(response['amount'])
        response['confirmations'] = int(response['confirmations'])


class APIUserTransactionsCall(APIPrivateCall):
    url = 'user_transactions/'

    def _process_response(self, response):
        for tx in response:
            tx['datetime'] = dt(tx['datetime'])
            tx['usd'] = Decimal(tx['usd'])
            tx['btc'] = Decimal(tx['btc'])
            tx['fee'] = Decimal(tx['fee'])


class APIWithdrawalRequestsCall(APIPrivateCall):
    url = 'withdrawal_requests/'

    def _process_response(self, response):
        for wr in response:
            wr['datetime'] = dt(wr['datetime'])
            wr['amount'] = Decimal(wr['amount'])



from setuptools import setup

setup(
    name='bitstampy',
    version='0.0.7',
    author='https://github.com/unwitting',
    author_email='jackprestonuk@gmail.com',
    packages=['bitstampy'],
    scripts=['bin/api_runthrough.py'],
    url='https://github.com/unwitting/bitstampy',
    license='LICENSE.txt',
    description='Bitstamp API wrapper for Python',
    long_description=open('README.txt').read(),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Topic :: Internet :: WWW/HTTP'
    ],
    install_requires=['requests']
)



from bitstampy import api
import time

# Incrementally call all non-private API endpoints

# Ticker
print('Ticker %s' % api.ticker())
time.sleep(1.1)

# Order Book
book = api.order_book()
book['bids'] = book['bids'][:5]
book['asks'] = book['asks'][:5]
print('Order book %s' % book)
time.sleep(1.1)

# Transactions
print('Transactions %s' % api.transactions(limit=5))
time.sleep(1.1)

# EUR / USD
print('EUR / USD conversion %s' % api.eur_usd_conversion_rate())
time.sleep(1.1)
